import React, { useState } from "react";
import { View, StyleSheet } from "react-native";
import { Calendar, LocaleConfig } from "react-native-calendars";
import { setupLocales } from "./locales";
import Icon from "react-native-vector-icons/MaterialIcons";
import dayjs from "dayjs";
import customParseFormat from "dayjs/plugin/customParseFormat";
import utc from "dayjs/plugin/utc";
import isSameOrBefore from "dayjs/plugin/isSameOrBefore";
import isSameOrAfter from "dayjs/plugin/isSameOrAfter";

/* We're not using some of these, but for less future bugs it doesn't hurt to
include them here */
dayjs.extend(customParseFormat);
dayjs.extend(utc);
dayjs.extend(isSameOrAfter);
dayjs.extend(isSameOrBefore);

//#region comment
/* A function with side effects. Mutates LocaleConfig object to have all
appropriate language locales. After this runs, you can be assured that setting
the locale for the calendar by running for example
  LocaleConfig.defaultLocale = "French";
will work. */
//#endregion
setupLocales();

/* TODO: Get current time as date string to be available as magic input in
default month to show input in Adalo. */
function MarkingsCalendar({
  isMarkingsPreviewEnabled,
  // Strings like "English", "Spanish", etc.
  calendarLanguage,
  // Can be "Sunday" or "Monday"
  firstDayOfWeek,
  onPressRangeStart,
  onPressRangeEnd,
  useToggle,
  // Props from colors section from Adalo editor
  /* The defaults for these props in the manifest.json are based off of the
  default styles from the react-native-calendars library which is in our file
  defaultStyles.js  */
  colors: {
    normalTextColor,
    fadedTextColor,
    backgroundColor,
    headingTextColor,
    dayOfWeekTextColor,
    todayTextColor,
    /* TODO: Allow Adalo users to set this for every marking in the same way as
    markingColor prop in the manifest. That way they have the option to use a
    very light or bright color marking with dark text for each individual
    marking if they want */
    markedTextColor,
    prevAndNextMonthIconColor,
    defaultDayMarkingColor,
    dotsColor,
    clickedColor,
    highlightTextColor
  },
  // Props from navigation section from Adalo editor
  previousMonthIconName,
  nextMonthIconName,
  previousAndNextMonthIconSize,
  /* Whether or not to show the arrows that let you go back or
    forward through the months in the calendar. */
  showArrows,
  onUnmarkedDayPressAction,
  onUnmarkedDayLongPressAction,
  /* The app-wide fonts from Adalo. At the moment there can only be 2 but
  that could change.
  Ex. of _fonts: {
    // for typical normal text
    body: "default",
    // for title text like the title in a screen's header bar
    heading: "Oswald"
  } */
  _fonts,
  /* The current height of the Adalo resizable container that is surrounding
  this component. The Adalo user can change this height in the Adalo editor, but
  thankfully any changes to this height doesn't affect the size or shape of this
  component at all. At worst it just crops your ability to view the component
  but that's it. */
  _height,
  // Tells us whether we're in the editor or not
  editor,
  // Derived props
  markings,
}) {
  const [inputIsDisabled, setInputIsDisabled] = useState(false);
  const [clickedDate, setClickedDate] = useState(undefined);

  function onDayPress(calendarDateObject) {
    const pressedDayNotInRange = !dateStringIsInRange({
      start: onPressRangeStart,
      date: calendarDateObject.dateString,
      end: onPressRangeEnd,
    });
    if (pressedDayNotInRange) {
      return;
    }

    // 'YYYY-MM-DD' ISO format
    const pressedDateString = calendarDateObject.dateString;
    //setClicked date
    if (useToggle) {

      if (pressedDateString === clickedDate) {
        setClickedDate(undefined);
        return;
      }
      else setClickedDate(pressedDateString);
    }
    // Longer ISO format in UTC. Ex: '2019-01-25T02:00:00.000Z'
    const pressedDateLongString = dayjs(pressedDateString).utc().toISOString();

    // Could be undefined in the case that user presses on an unmarked day
    const pressedDayMarking = markings[pressedDateString];
    const pressedDayIsMarked = Boolean(pressedDayMarking?.markingColor);

    /* The gist of this is simple: if pressed day is marked then run the marked
    day press action. Otherwise, the pressed day is unmarked so run the unmarked
    day press action. That's it. */
    if (
      pressedDayIsMarked &&
      typeof pressedDayMarking.onDayPressAction === "function"
    ) {
      /* Since the pressed day is marked, we have to run the marked day
      press action with 2 args, marking's date & color */
      const { onDayPressAction, markingColor } = pressedDayMarking;

      /* This part is just running the action with some input disabling. I'm
      temporarily disabling input because database changes from action could be
      slow & I need to prevent quick user presses to avoid bugs in this
      scenario */
      setInputIsDisabled(true);
      onDayPressAction(pressedDateLongString, markingColor).then(
        // On success
        () => {
          setInputIsDisabled(false);
        },
        // On failure
        (error) => {
          setInputIsDisabled(false);
          console.error("action had an error while running:", error);
        }
      );
    } else if (
      !pressedDayIsMarked &&
      typeof onUnmarkedDayPressAction === "function"
    ) {
      /* The day is unmarked so just run the unmarked day press action with some
     input disabling to prevent some user press bugs */
      setInputIsDisabled(true);
      onUnmarkedDayPressAction(pressedDateLongString).then(
        // On success
        () => {
          setInputIsDisabled(false);
        },
        // On failure
        (error) => {
          setInputIsDisabled(false);
          console.error("action had an error while running:", error);
        }
      );
    }
  }

  function onDayLongPress(calendarDateObject) {
    const pressedDayNotInRange = !dateStringIsInRange({
      start: onPressRangeStart,
      date: calendarDateObject.dateString,
      end: onPressRangeEnd,
    });
    if (pressedDayNotInRange) {
      return;
    }

    // 'YYYY-MM-DD' ISO format
    const pressedDateString = calendarDateObject.dateString;
    //setClicked date
    if (useToggle) {

      if (pressedDateString === clickedDate) {
        setClickedDate(undefined);
        return;
      }
    }
    else setClickedDate(pressedDateString);
    // Longer ISO format in UTC. Ex: '2019-01-25T02:00:00.000Z'
    const pressedDateLongString = dayjs(pressedDateString).utc().toISOString();

    // Could be undefined in the case that user presses on an unmarked day
    const pressedDayMarking = markings[pressedDateString];
    const pressedDayIsMarked = Boolean(pressedDayMarking?.markingColor);

    /* The gist of this is simple: if pressed day is marked then run the marked
    day press action. Otherwise, the pressed day is unmarked so run the unmarked
    day press action. That's it. */
    if (
      pressedDayIsMarked &&
      typeof pressedDayMarking.onDayLongPressAction === "function"
    ) {
      /* Since the pressed day is marked, we have to run the marked day
      press action with 2 args, marking's date & color */
      const { onDayLongPressAction, markingColor } = pressedDayMarking;

      /* This part is just running the action with some input disabling. I'm
      temporarily disabling input because database changes from action could be
      slow & I need to prevent quick user presses to avoid bugs in this
      scenario */
      setInputIsDisabled(true);
      onDayLongPressAction(pressedDateLongString, markingColor).then(
        // On success
        () => {
          setInputIsDisabled(false);
        },
        // On failure
        (error) => {
          setInputIsDisabled(false);
          console.error("action had an error while running:", error);
        }
      );
    } else if (
      !pressedDayIsMarked &&
      typeof onUnmarkedDayLongPressAction === "function"
    ) {
      /* The day is unmarked so just run the unmarked day press action with some
     input disabling to prevent some user press bugs */
      setInputIsDisabled(true);
      onUnmarkedDayLongPressAction(pressedDateLongString).then(
        // On success
        () => {
          setInputIsDisabled(false);
        },
        // On failure
        (error) => {
          setInputIsDisabled(false);
          console.error("action had an error while running:", error);
        }
      );
    }
  }
  function createClickedDate(date) {
    let dateObj = {};
    if (!date || !useToggle) return dateObj;
    dateObj[date] = {
      selectedColor: clickedColor,
      selectedTextColor: highlightTextColor,
      selected: true,
      marked: true,
      startingDay: true,
      endingDay: true,
      dots: markedDates[date] ? markedDates[date].dots : []
    }
    return dateObj;
  }

  function createMarkedDates(markings) {
    if (Object.keys(markings).length === 0) return {};

    const markedDates = {};
    for (const [dateString, marking] of Object.entries(markings)) {
      markedDates[dateString] = {
        selectedColor: marking.markingColor,
        selectedTextColor: markedTextColor,
        selected: true,
        marked: true,
        startingDay: true,
        endingDay: true,
        dots: createDots(marking.eventString, dotsColor)
      };
    }
    /* TODO: Add Adalo switch in manifest that let's you set that you want
    markings with the same colors to attract. To do that, you should adjust &
    use commented out function below. */
    // fixMarkedDatesAttractionValues(markedDates);
    return markedDates;
  }

  //#region commented out code to possibly use in future
  // if having the colors attract when the colors are the same becomes a desired
  // feature
  //
  // function fixMarkedDatesAttractionValues(markedDates) {
  //   // Avoiding O(n^2) time by mutating instead of copying.
  //   for (const dateString in markedDates) {
  //     // Runs in O(1) time. Copying would've made it O(n) time instead
  //     fixMarkedDatesAttractionValuesForAGivenDay(markedDates, dateString);
  //   }
  // }

  /* Make sure this func stays at O(1) time. Mutating object instead of copying
  means we can do this in constant O(1) time instead of O(n) which avoids O(n^2)
  times in other algorithms in this component. */
  // function fixMarkedDatesAttractionValuesForAGivenDay(
  //   markedDates,
  //   currDateString
  // ) {
  //   const prevDateString = dayjs(currDateString)
  //     .subtract(1, "day")
  //     .format("YYYY-MM-DD");
  //   const nextDateString = dayjs(currDateString)
  //     .add(1, "day")
  //     .format("YYYY-MM-DD");

  //   const prevDateColor = markedDates[prevDateString]?.color || null;
  //   const currDateColor = markedDates[currDateString]?.color || null;
  //   const nextDateColor = markedDates[nextDateString]?.color || null;

  //   const prevAndCurrDateAreAttracted = colorsAreAttracted(
  //     prevDateColor,
  //     currDateColor
  //   );
  //   const currAndNextDateAreAttracted = colorsAreAttracted(
  //     currDateColor,
  //     nextDateColor
  //   );

  //   const prevDayHasMarking = isAValidDayMarkingColor(prevDateColor);
  //   if (prevDayHasMarking) {
  //     markedDates[prevDateString] = {
  //       ...markedDates[prevDateString],
  //       endingDay: !prevAndCurrDateAreAttracted,
  //     };
  //   }

  //   const currDayHasMarking = isAValidDayMarkingColor(currDateColor);
  //   if (currDayHasMarking) {
  //     markedDates[currDateString] = {
  //       ...markedDates[currDateString],
  //       startingDay: !prevAndCurrDateAreAttracted,
  //       endingDay: !currAndNextDateAreAttracted,
  //     };
  //   }

  //   const nextDayHasMarking = isAValidDayMarkingColor(nextDateColor);
  //   if (nextDayHasMarking) {
  //     markedDates[nextDateString] = {
  //       ...markedDates[nextDateString],
  //       startingDay: !currAndNextDateAreAttracted,
  //     };
  //   }
  // }

  /* Attracted means the colors want to be 'joined' rather than 'disconnected'.
  Used for deciding whether the calendar's color markings should combine or not
  if they're next to each other. */
  // function colorsAreAttracted(color1, color2) {
  //   return color1 === "#00FF00" && color2 === "#00FF00";
  // }
  //#endregion

  /* Things to do before rendering
    ==================================================================== */
  /* With this, now the calendar will automatically be configured with all the
  appropriate strings like the day & month names for the given language */
  LocaleConfig.defaultLocale = calendarLanguage;

  const markedDates = createMarkedDates(markings);

  /* This should match with the default height in the manifest */
  const BASE_HEIGHT = 400;
  const { weekMarginVertical } = calcStylesForDynamicHeightPurposes(
    _height,
    BASE_HEIGHT
  );

  // console.log("RENDER HAPPENED WEEEEEEEE WOOOOOOOO WEEEEEEEE WOOOOOOOO");
  console.log("markings", markings);
  console.log("markedDates", markedDates);
  console.log(dayjs().format("YYYY-MM-DD"));
  return (
    <View style={styles.container}>
      {
        <Calendar
          // A 'trick' that forces a complete re-render when changing these
          // variables, where otherwise changing these variables would not cause a
          // re-render & therefore the Adalo user wouldn't see any of their
          // changes in the Adalo editor. Changing these variables will change the
          // string for this key, which'll then cause the re-render. Key must be
          // unique across all components in the app, which is why I put a unique
          // enough prefix.
          key={
            //#region creation of the key string
            "MarkingsCalendar." +
            firstDayOfWeek +
            calendarLanguage +
            // While technically not needed, leave it here just in case.
            String(showArrows) +
            previousMonthIconName +
            nextMonthIconName +
            previousAndNextMonthIconSize +
            prevAndNextMonthIconColor +
            normalTextColor +
            fadedTextColor +
            backgroundColor +
            headingTextColor +
            dayOfWeekTextColor +
            todayTextColor +
            markedTextColor +
            highlightTextColor +
            _height
            //#endregion
          }
          // Some properties are commented out so that Calendar can just give it
          // the default. Don't delete them so you can see what properties are
          // available to you.
          // TODO: Let people choose the font weight for the day, day
          // of the week, and month text. Make a note that the font needs to
          // support their weight in order for it work.
          theme={{
            calendarBackground: backgroundColor,
            // Color for day of the week text
            textSectionTitleColor: dayOfWeekTextColor,
            // Selected day happens when a day has a 'selected: true' marking
            // selectedDayBackgroundColor: defaultStyle.selectedDayBackgroundColor,
            todayTextColor: todayTextColor,
            indicatorColor: inputIsDisabled ? prevAndNextMonthIconColor : "rgba(255, 255, 255, 0)",
            dayTextColor: normalTextColor,
            monthTextColor: headingTextColor,
            textDisabledColor: fadedTextColor,
            // dotColor: defaultStyle.dotColor,
            selectedDotColor: backgroundColor,

            // Thae 300 font weight looks noticeably much better than the regular
            // 400, assuming the font has that weight available of course.
            // textDayFontWeight: "300",
            // textMonthFontWeight: "300",
            // textDayHeaderFontWeight: "300", // undefined by default
            // textDayFontSize: 16,
            // textMonthFontSize: 16,
            // textDayHeaderFontSize: 13,

            // Way it works is the normal way setting font family works in React
            // Native CSS. You set font family to a string & the css will lookup
            // that string identifier in its installed fonts. _font.body is
            // "default" for the default system font but you don't have to worry
            // about all that since you can just pass the string & Adalo will make
            // sure the string identifier & the font the app has installed will
            // match.
            textDayFontFamily: _fonts.body,
            textMonthFontFamily: _fonts.body,
            "stylesheet.calendar.main": {
              week: {
                marginTop: weekMarginVertical,
                marginBottom: weekMarginVertical,
                flexDirection: "row",
                justifyContent: "space-around",
              },
            },
          }}
          // 0 means start the week on Sunday. 1 for Monday. Not sure why the
          // prop does it like that.
          firstDay={firstDayOfWeek === "Sunday" ? 0 : 1}
          hideArrows={!showArrows}
          renderArrow={(direction) =>
            direction === "left" ? (
              <Icon
                name={previousMonthIconName}
                size={previousAndNextMonthIconSize}
                color={prevAndNextMonthIconColor}
              />
            ) : (
              <Icon
                name={nextMonthIconName}
                size={previousAndNextMonthIconSize}
                color={prevAndNextMonthIconColor}
              />
            )
          }
          // Sets the default month to show on first load & whenever key changes
          current={dayjs().format("YYYY-MM-DD")}
          displayLoadingIndicator={true}
          // Set to true so that month won't change when user presses an extra day
          // from another month. Which leads to a more consistent UX.
          disableMonthChange={true}
          onDayPress={inputIsDisabled ? undefined : onDayPress}
          onDayLongPress={inputIsDisabled ? undefined : onDayLongPress}
          // Can potentially prevent weird bugs where a month will get skipped
          // on an arrow press
          onPressArrowLeft={(subtractMonth) => subtractMonth()}
          onPressArrowRight={(addMonth) => addMonth()}
          // Markings
          markedDates={
            editor === true && isMarkingsPreviewEnabled
              ? createSampleMarkedDates(defaultDayMarkingColor, markedTextColor, dotsColor)
              : { ...markedDates, ...createClickedDate(clickedDate) }
          }
          markingType={"multi-dot"}
        />

      }
    </View>
  );
}

const styles = StyleSheet.create({
  /* I'm not sure why we need flex or margin top, or even why we would need to
  wrap Calendar in a View but I'm putting this CSS here just to be safe since
  Adalo's calendar had it to maybe avoid bugs, I don't know. */
  container: {
    flex: 1,
    // Try adding back in if you have UI problems
    // marginTop: 5,
  },
});

function createDots(events, dotsColor) {
  let dots = []
  for (let i = 0; i < events; i++) {
    dots.push({ color: dotsColor });
  }
  return dots;
}

function createSampleMarkedDates(markingColor, markedTextColor, dotsColor) {

  const todaysDate = dayjs();

  /* dayjs get() outputs a number. Getting the month starts at 0 so we need to
  add 1 */
  const todaysMonth = (todaysDate.get("month") + 1).toString().padStart(2, "0");
  const todaysYear = todaysDate.get("year").toString();
  return {
    [`${todaysYear}-${todaysMonth}-01`]: {
      selectedColor: markingColor,
      selectedTextColor: markedTextColor,
      selected: true,
      marked: true,
      startingDay: true,
      endingDay: true,
      dots: [{ color: dotsColor }, { color: dotsColor }, { color: dotsColor }]
    },
    [`${todaysYear}-${todaysMonth}-03`]: {
      selected: true,
      marked: true,
      selectedColor: markingColor,
      selectedTextColor: markedTextColor,
      startingDay: true,
      endingDay: true,
      dots: [{ color: dotsColor }, { color: dotsColor }]
    },
    [`${todaysYear}-${todaysMonth}-07`]: {
      selected: true,
      marked: true,
      selectedColor: markingColor,
      selectedTextColor: markedTextColor,
      startingDay: true,
      endingDay: true,
      dotColor: 'red'
    },
    [`${todaysYear}-${todaysMonth}-08`]: {
      selected: true,
      marked: true,
      selectedColor: markingColor,
      selectedTextColor: markedTextColor,
      startingDay: true,
      endingDay: true,
      dotColor: 'red'
    },
  };
}

function calcStylesForDynamicHeightPurposes(_height, BASE_HEIGHT) {
  const increasedHeightFromBase = Math.max(_height - BASE_HEIGHT, 0);
  const decreasedHeightFromBase = Math.max(BASE_HEIGHT - _height, 0);

  let weekMarginVertical = 7; // actual default from Calendar
  if (decreasedHeightFromBase > 0) {
    weekMarginVertical = Math.max(
      weekMarginVertical - decreasedHeightFromBase / 12,
      0
    );
  } else if (increasedHeightFromBase > 0) {
    weekMarginVertical += increasedHeightFromBase / 12;
  }

  console.log("_height", _height);
  // console.log("increasedHeightFromBase", increasedHeightFromBase);
  // console.log("decreasedHeightFromBase", decreasedHeightFromBase);
  // console.log("weekMarginVertical", weekMarginVertical);

  return { weekMarginVertical };
}

// Inclusively checking if date is between two other dates. Start & end being
// null just means that that boundary doesn't exist.
function dateStringIsInRange({
  start: startDateString,
  date: dateString,
  end: endDateString,
}) {
  const dateIsSameOrAfterLeftBound =
    !startDateString || dayjs(dateString).isSameOrAfter(startDateString);
  const dateIsSameOrBeforeRightBound =
    !endDateString || dayjs(dateString).isSameOrBefore(endDateString);
  return dateIsSameOrAfterLeftBound && dateIsSameOrBeforeRightBound;
}

export { MarkingsCalendar };
