import React from "react";
import dayjs from "dayjs";
import customParseFormat from "dayjs/plugin/customParseFormat";
import utc from "dayjs/plugin/utc";
import { MarkingsCalendar } from "./MarkingsCalendar";

dayjs.extend(customParseFormat);
dayjs.extend(utc);

/* TODO: Get current time as date string to be available as magic input in
default month to show input in Adalo. */
function AdaloMarkingsCalendar({
  adaloInputMarkings,
  onUnmarkedDayPressAction,
  onUnmarkedDayLongPressAction,
  isMarkingsPreviewEnabled,
  // Strings like "English", "Spanish", etc.
  calendarLanguage,
  // Can be "Sunday" or "Monday"
  firstDayOfWeek,
  enableOnPressRangeStart,
  enableOnPressRangeEnd,
  onPressRangeStart,
  onPressRangeEnd,
  useToggle,
  // Props from colors section from Adalo editor
  /* The defaults for these props in the manifest.json are based off of the
  default styles from the react-native-calendars library which is in our file
  defaultStyles.js  */
  /* TODO: Make default object to cover case where color'll sometimes be
  undefined when updating an already existing component to have another prop. Do
  this for all other props as well since it affects most props really. */
  colors: {
    normalTextColor,
    fadedTextColor,
    backgroundColor,
    headingTextColor,
    dayOfWeekTextColor,
    todayTextColor,
    markedTextColor,
    prevAndNextMonthIconColor,
    defaultDayMarkingColor,
    clickedColor,
    dotsColor,
    highlightTextColor,
  },
  // Props from navigation section from Adalo editor
  navigation: {
    previousMonthIconName,
    nextMonthIconName,
    previousAndNextMonthIconSize,
    /* Whether or not to show the arrows that let you go back or
    forward through the months in the calendar. */
    showArrows,
  },
  /* The app-wide fonts from Adalo. At the moment there can only be 2 but
  that could change.
  Ex. of _fonts: {
    // for typical normal text
    body: "default",
    // for title text like the title in a screen's header bar
    heading: "Oswald"
  } */
  _fonts,
  /* The current height of the Adalo resizable container that is surrounding
  this component. The Adalo user can change this height in the Adalo editor, but
  thankfully any changes to this height doesn't affect the size or shape of this
  component at all. At worst it just crops your ability to view the component
  but that's it. */
  _height,
  editor,
}) {
  console.log("adaloInputMarkings b4 cleaning", adaloInputMarkings);
  // console.log("onPressRangeStart b4", onPressRangeStart);
  // console.log("onPressRangeEnd b4", onPressRangeEnd);
  // console.log("onMarkingsChangeAction b4", onMarkingsChangeAction);

  /* Cleaning props
    ==================================================================== */
  //#region comment
  /* Cleaning props data that could use it. There is a contract in this
  component that any props data is assumed to be well-behaved and clean, so this
  cleaning is necessary to uphold that. Benefit is not worrying about weird
  input from Adalo. */
  //#endregion
  onPressRangeStart = cleanupOnPressRangeStart(onPressRangeStart);
  onPressRangeEnd = cleanupOnPressRangeEnd(onPressRangeEnd);
  previousAndNextMonthIconSize = cleanupPreviousAndNextMonthIconSize(
    previousAndNextMonthIconSize
  );
  adaloInputMarkings = cleanupAdaloInputMarkings(
    adaloInputMarkings,
    defaultDayMarkingColor
  );
  onUnmarkedDayPressAction = cleanupOnUnmarkedDayPressAction(
    onUnmarkedDayPressAction
  );
  onUnmarkedDayLongPressAction = cleanupOnUnmarkedDayLongPressAction(
    onUnmarkedDayLongPressAction
  );
  isMarkingsPreviewEnabled =
    isMarkingsPreviewEnabled === true || isMarkingsPreviewEnabled === false
      ? isMarkingsPreviewEnabled
      : true;
  onPressRangeStart = enableOnPressRangeStart ? onPressRangeStart : null;
  onPressRangeEnd = enableOnPressRangeEnd ? onPressRangeEnd : null;

  console.log("adaloInputMarkings after cleaning", adaloInputMarkings);

  /* Making derived props
    ==================================================================== */
  /* Use this instead of adalo's input markings prop. It's supposed to
  replace it as a better data format. */
  const markings = createMarkings(adaloInputMarkings);
  /* Other things to do before rendering
    ==================================================================== */
  // console.log("markings", markings);
  // console.log("onPressRangeStart after", onPressRangeStart);
  // console.log("onPressRangeEnd after", onPressRangeEnd);
  return (
    <MarkingsCalendar
      // Cleaned up Adalo props
      isMarkingsPreviewEnabled={isMarkingsPreviewEnabled}
      firstDayOfWeek={firstDayOfWeek}
      calendarLanguage={calendarLanguage}
      onPressRangeStart={onPressRangeStart}
      onPressRangeEnd={onPressRangeEnd}
      useToggle={useToggle}
      colors={{
        normalTextColor,
        fadedTextColor,
        backgroundColor,
        headingTextColor,
        dayOfWeekTextColor,
        todayTextColor,
        markedTextColor,
        prevAndNextMonthIconColor,
        defaultDayMarkingColor,
        dotsColor,
        clickedColor,
        highlightTextColor
      }}
      previousMonthIconName={previousMonthIconName}
      nextMonthIconName={nextMonthIconName}
      previousAndNextMonthIconSize={previousAndNextMonthIconSize}
      showArrows={showArrows}
      onUnmarkedDayPressAction={onUnmarkedDayPressAction}
      onUnmarkedDayLongPressAction={onUnmarkedDayLongPressAction}
      _fonts={_fonts}
      _height={_height}
      editor={editor}
      // Derived props
      markings={markings}
    />
  );
}

/* Basically, we're just turning our array into an object by setting the keys to
what the date string is & immutably removing the date string from our markings
object.
Ex. In:
[
  {
    // Guaranteed valid date string
    dateString: '2021-02-15',
    // Guaranteed to always be a color in format #RRGGBB
    markingColor,
    onDayPressAction,
    onDayLongPressAction,
    // other entries like _meta and id
  }
]

Ex. Out:
{
  '2021-02-15': {
    markingColor,
    onDayPressAction,
    onDayLongPressAction,
    // other entries like _meta and id
  }
} 
*/
function createMarkings(adaloInputMarkings) {
  if (adaloInputMarkings.length === 0) return {};

  const markings = {};
  let currDateString = null;
  let newInputMarking = {};
  for (const inputMarking of adaloInputMarkings) {
    currDateString = inputMarking.dateString;

    newInputMarking = { ...inputMarking };
    delete newInputMarking.dateString;

    markings[currDateString] = newInputMarking;
  }
  return markings;
}

/* Cleanup so that it'll always be an array, either empty or filled with only
objects with valid date strings. And result'll have only 1 marking per date
string, no duplicates. For duplicates, the first one in the array overwrites all
the other ones.
Ex. In:
[
  {
    ISODateString,
    markingColorCategory: { markingColor },
    onMarkedDayPressAction,
    onMarkedDayLongPressAction,
    // other entries like _meta and id
  }
]

Ex. Out: 
[
  {
    // Guaranteed valid date string
    dateString,
    // Guaranteed to always be a color in format #RRGGBB
    markingColor,
    onDayPressAction,
    onDayLongPressAction,
    // other entries like _meta and id
  }
]
with duplicates removed
*/
function cleanupAdaloInputMarkings(adaloInputMarkings, defaultDayMarkingColor) {
  // Condition translates to: array does not exist, is not an array, or is empty
  if (!Array.isArray(adaloInputMarkings) || adaloInputMarkings.length === 0) {
    return [];
  }
  const seenDateStrings = new Set();
  // Cleaning up our markings
  adaloInputMarkings = adaloInputMarkings
    /* Filter so that all elements are objects with a string that has a valid
    ISO 8601 format */
    .filter((marking) => {
      const ISODateString = marking?.ISODateString;
      return isValidISO8601DateString(ISODateString);
    })
    /* Convert the local time zone ISO date string to UTC time zone, since UTC
    is how Adalo seems to stores the time in the database, but in the props they
    give the ISO date in the device's local time.
    After that, then convert that to YYYY-MM-DD date string, which is the format
    we use for date strings in our code */
    .map((marking) => {
      let {
        ISODateString,
        onMarkedDayPressAction: onDayPressAction,
        onMarkedDayLongPressAction: onDayLongPressAction,
        ...rest
      } = marking;

      const dateString = dayjs(ISODateString).utc().format("YYYY-MM-DD");

      let markingColor = defaultDayMarkingColor;
      onDayPressAction =
        typeof onDayPressAction === "function" ? onDayPressAction : null;
      onDayLongPressAction =
        typeof onDayLongPressAction === "function"
          ? onDayLongPressAction
          : null;

      marking = {
        ...rest,
        dateString,
        markingColor,
        onDayPressAction,
        onDayLongPressAction,
      };
      return marking;
    })
    /* Get rid of duplicates, meaning object elements that have the same date
    string */
    .filter((marking) => {
      const { dateString } = marking;
      let isMarkingARepeat = null;
      if (seenDateStrings.has(dateString)) {
        isMarkingARepeat = true;
      } else {
        isMarkingARepeat = false;
        seenDateStrings.add(dateString);
      }
      return !isMarkingARepeat;
    });
  return adaloInputMarkings;
}

function cleanupMarkingColor(markingColor, defaultDayMarkingColor) {
  if (typeof markingColor !== "string") return defaultDayMarkingColor;

  // Ex. of valid string: #00FF00
  const validRgbHexFormatRegex = /^#[0-9a-fA-F]{6}$/;
  const markingColorIsValid = validRgbHexFormatRegex.test(markingColor);

  markingColor = markingColorIsValid ? markingColor : defaultDayMarkingColor;
  return markingColor;
}

function cleanupOnUnmarkedDayPressAction(onUnmarkedDayPressAction) {
  return typeof onUnmarkedDayPressAction === "function"
    ? onUnmarkedDayPressAction
    : null;
}

function cleanupOnUnmarkedDayLongPressAction(onUnmarkedDayLongPressAction) {
  return typeof onUnmarkedDayLongPressAction === "function"
    ? onUnmarkedDayLongPressAction
    : null;
}

/* The date from this is not converted to UTC since it's seems to me pretty
obvious that the only input an Adalo user would ever put in here is the 'date &
time' magic data type. In that case, I saw from my experimenting that you should
not convert that time to UTC. */
function cleanupOnPressRangeStart(onPressRangeStart) {
  return isValidISO8601DateString(onPressRangeStart)
    ? dayjs(onPressRangeStart).format("YYYY-MM-DD")
    : null;
}

/* The date from this is not converted to UTC since it's seems to me pretty
obvious that the only input an Adalo user would ever put in here is the 'date &
time' magic data type. In that case, I saw from my experimenting that you should
not convert that time to UTC. */
function cleanupOnPressRangeEnd(onPressRangeEnd) {
  // By default don't allow short presses after current day
  return isValidISO8601DateString(onPressRangeEnd)
    ? dayjs(onPressRangeEnd).format("YYYY-MM-DD")
    : dayjs().format("YYYY-MM-DD");
}

function cleanupPreviousAndNextMonthIconSize(previousAndNextMonthIconSize) {
  /* Make sure all this code matches up with what's in the manifest. Adalo's
  slider manifest min & max settings lie to you & allow input beyond the limits
  set. So you need to set the damn limits yourself here. */
  previousAndNextMonthIconSize =
    typeof previousAndNextMonthIconSize === "number"
      ? previousAndNextMonthIconSize
      : 27;
  previousAndNextMonthIconSize = Math.max(previousAndNextMonthIconSize, 1);
  previousAndNextMonthIconSize = Math.min(previousAndNextMonthIconSize, 38);
  return previousAndNextMonthIconSize;
}

function isValidISO8601DateString(dateString) {
  if (typeof dateString !== "string" || dateString.length === 0) {
    return false;
  }

  // Regex from: https://stackoverflow.com/a/14322189/13255028
  // Checks if it's ANY valid ISO8601 format, not just the long version
  const re =
    // eslint-disable-next-line no-useless-escape
    /^([\+-]?\d{4}(?!\d{2}\b))((-?)((0[1-9]|1[0-2])(\3([12]\d|0[1-9]|3[01]))?|W([0-4]\d|5[0-2])(-?[1-7])?|(00[1-9]|0[1-9]\d|[12]\d{2}|3([0-5]\d|6[1-6])))([T\s]((([01]\d|2[0-3])((:?)[0-5]\d)?|24\:?00)([\.,]\d+(?!:))?)?(\17[0-5]\d([\.,]\d+)?)?([zZ]|([\+-])([01]\d|2[0-3]):?([0-5]\d)?)?)?)?$/;
  const dateStringIsValid =
    re.test(dateString) &&
    // Maybe not needed but hey it can't hurt
    dayjs(
      dateString,
      undefined,
      // Making strict checking true
      true
    ).isValid();
  return dateStringIsValid;
}

export default AdaloMarkingsCalendar;
